#!/usr/bin/perl

# Author : Damien KROTKINE (damien@tuxfamily.org)
#
# Contributors :
#
# Copyright (C) 2002 Damien KROTKINE (damien@tuxfamily.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package Libconf::Glueconf::XF86Config;
use strict;
use vars qw(@ISA);
use Libconf;
#use Libconf::Glueconf;
#use Libconf::Glueconf::XF86Config::Wrapper;
@ISA = qw(Libconf);

my @keywords_option = ();
my @keywords_list = ();

sub new {
    my ($class, $filename) = @_;
    my $libconf = Libconf::new('Libconf', $filename, 'XF86Config', '');
#    print Data::Dumper->Dump([$libconf],['libconf']) . "\n";
    #my $ret = Libconf::Glueconf::generateFunctions(@synopsis);
    #print " plop : $ret\n";
    #eval Libconf::Glueconf::generateFunctions(@synopsis);
    #$@ and die $@;
#    $self->{content} = {};
    tie my %wrapper, 'Libconf::Glueconf::XF86Config::Wrapper', $libconf;
#    print Data::Dumper->Dump([%wrapper],['wrapper']) . "\n";
    bless \%wrapper, $class;
}

#sub getSections {
#    my $self = shift;
#    map { my $atom = $self->getAtom($_); $atom->{section_name} } ($self->findAtomPos({type => 'SECTION'}));
#}


package Libconf::Glueconf::XF86Config::Wrapper;

sub TIEHASH {
    my ($pkg, $libconf) = @_;
    print "Wrapper - TIEHASH\n";
    bless { libconf => $libconf }, $pkg;
}

sub CLEAR {
    my ($obj) = @_;
    print "Wrapper - CLEAR\n";
    $obj->{libconf}->clear;
}

#sub DESTROY {
#}

sub DELETE {
    my ($obj, $key) = @_;
    print "Wrapper - DELETE - key: $key\n";
    my @pos = $obj->{libconf}->findAtomPos( { type => 'SECTION', sections => [], section_name => $key });
    foreach (@pos) {
        $obj->{libconf}->deleteSection($_);
    }
}

sub FIRSTKEY {
    my ($obj) = @_;
    print "Wrapper - FIRSTKEY\n";
    my $atom = $obj->{libconf}->getAtom(0);
    $atom->{type} eq 'SECTION' or die "atom n 0 is not a section\n";
    $atom->{section_name};
}

sub EXISTS {
    my ($obj, $key) = @_;
    print "Wrapper - EXISTS - key : $key\n";
    my $pos = $obj->{libconf}->findAtomPos( { type => 'SECTION', sections => [], section_name => $key });
    defined $pos;
}

sub NEXTKEY {
    my ($obj, $lastkey) = @_;
    print "Wrapper - NEXTKEY - lastkey : $lastkey\n";
    my $pos = $obj->{libconf}->findAtomPos( { type => 'SECTION', sections => [], section_name => $lastkey });
    $obj->{libconf}->getAtom($pos+1)->{section_name};
}

sub STORE {
    my ($obj, $key, $value) = @_;
    print "Wrapper - STORE - key : $key - value : $value\n";
    ref $value eq 'HASH' or die 'trying to store anything else than hash ref';
    my $index;
    my @pos = $obj->{libconf}->findAtomPos({ type => 'SECTION', section_name => $key, sections => [] });
    if (@pos == 0) {
        $index = $obj->{libconf}->appendAtom({ section_name => $key, type => 'SECTION' });
    } else {
        $index = $pos[0];
    }
    my %hash;
    tie %hash, 'Libconf::Glueconf::XF86Config::SectionWrapper', $obj->{libconf}, $index;
    %hash = %$value;
}

sub FETCH {
    my ($obj, $key) = @_;
    print "Wrapper - FETCH - key : $key\n";
    $key eq 'libconf' and return $obj->{libconf};
    my @section_positions = $obj->{libconf}->findAtomPos( { type => 'SECTION', section_name => $key, sections => [ ] });
    my %ret;
    if (Libconf::member($key,
                        'InputDevice',
                        'Device',
                        'Monitor',
                        'Modes',
                        'Screen',
                        'ServerLayout',
                        'Vendor',
                       )) {
        my @identifier_positions = $obj->{libconf}->findAtomPos(
                                                                { type => 'KEY_VALUE',
                                                                  key => 'Identifier',
                                                                  sections => [ {name => $key} ]
                                                                }
                                                               );
        my $i = 0;
        foreach (@identifier_positions) {
            my $atom = $obj->{libconf}->getAtom($_);
            $ret{$atom->{value}} = {};
            tie %{$ret{$atom->{value}}}, 'Libconf::Glueconf::XF86Config::SectionWrapper', $obj->{libconf}, $section_positions[$i++];
        }
        return \%ret;
    }
    @section_positions > 1 and die "there are multiple sections named $key, but I don't know how to handle them";
    tie %ret, 'Libconf::Glueconf::XF86Config::SectionWrapper', $obj->{libconf}, $section_positions[0];
    \%ret;
}

package Libconf::Glueconf::XF86Config::SectionWrapper;

sub TIEHASH {
    my ($pkg, $libconf, $beginning) = @_;
    print "SectionWrapper - TIEHASH - beginning : $beginning\n";
    my $atom = $libconf->getAtom($beginning);
    my @depth = @{$atom->{sections}};
    push @depth, { name => $atom->{section_name} };

    my $key = $beginning+1;
    foreach ($beginning+1..$libconf->size-1) {
        my $atom = $libconf->getAtom($_);
        if (!(@{$atom->{sections}} >= @depth)) {
            $key++;
            next;
        }
        Libconf::compare_depth($atom->{sections}, \@depth) or last;
        $key++;
    }
    my %hash = (
                libconf => $libconf,
                firstatom => $beginning,
                sections => \@depth,
                lastatom => $key,
                Options => {},
                Load => {},
               );
#    print Data::Dumper->Dump([%hash],['hash']) . "\n";
    print "SectionWrapper - TIEHASH - firstatom : $hash{firstatom} - lastatom : $hash{lastatom} \n";
    return bless \%hash, $pkg;
#     bless {
#            libconf => $libconf,
#            firstatom => $beginning,
#            sections => \@depth,
#            lastatom => $key,
#           }, $pkg;
}

sub CLEAR {
    my ($obj) = @_;
    print "SectionWrapper - CLEAR\n";
    $obj->{lastatom} -= $obj->{libconf}->clearSection($obj->{firstatom});
    $obj->{lastatom} == $obj->{firstatom} + 1 or die "removed atoms number is wrong";
#    $obj->{lastatom} = $obj->{firstatom} + 1;
}

#sub DESTROY {
#}

sub DELETE {
    my ($obj, $key) = @_;
    print "SectionWrapper - DELETE - key : $key\n";
    foreach ($obj->{firstatom}+1..$obj->{lastatom}-1) {
        my $atom = $obj->{libconf}->getAtom($_);
        if ($atom->{type} eq 'SECTION' &&
            $atom->{section_name} eq $key) {
            my $removed = $obj->{libconf}->deleteSection($_);
            $obj->{lastatom} -= $removed;
            last;
        } elsif ($atom->{type} eq 'KEY_VALUE' &&
                 $atom->{key} eq $key) {
            $obj->{libconf}->deleteAtom($_);
            $obj->{lastatom}--;
        } elsif ($atom->{type} eq 'VALUE' &&
                 $atom->{value} eq $key) {
            $obj->{libconf}->deleteAtom($_);
            $obj->{lastatom}--;
        }
    }
}

sub FIRSTKEY {
    my ($obj) = @_;
    print "SectionWrapper - FIRSTKEY\n";
    $obj->{firstatom} + 1 == $obj->{lastatom}and return undef;
    my $atom = $obj->{libconf}->getAtom($obj->{firstatom}+1);
    $atom->{type} eq 'SECTION' and return $atom->{section_name};
    $atom->{type} eq 'KEY_VALUE' and return $atom->{key};
    $atom->{type} eq 'VALUE' and return $atom->{value};
    die "houston, we have a problem";
}

sub EXISTS {
    my ($obj, $key) = @_;
    print "SectionWrapper - EXISTS - key : $key\n";
    foreach ($obj->{firstatom}+1..$obj->{lastatom}-1) {
        my $atom = $obj->{libconf}->getAtom($_);
        ($atom->{type} eq 'SECTION' && $atom->{section_name} eq $key) ||
        ($atom->{type} eq 'KEY_VALUE' && $atom->{key} eq $key) and return 1;
        ($atom->{type} eq 'VALUE' && $atom->{value} eq $key) and return 1;
    }
}

sub NEXTKEY {
    my ($obj, $lastkey) = @_;
    print "SectionWrapper - NEXTKEY - lastkey : $lastkey\n";
    foreach ($obj->{firstatom}+1..$obj->{lastatom}-1) {
        my $atom = $obj->{libconf}->getAtom($_);
        if ($atom->{type} eq 'SECTION' && $atom->{section_name} eq $lastkey) {
            $_+1 == $obj->{lastatom} and return undef;
            return $obj->{libconf}->getAtom($_+1)->{section_name};
        }
        if ($atom->{type} eq 'KEY_VALUE' && ! $atom->{type2} && $atom->{key} eq $lastkey) {
            $_+1 == $obj->{lastatom} and return undef;
            return $obj->{libconf}->getAtom($_+1)->{key};
        }
        if ($atom->{type} eq 'VALUE' && ! $atom->{type2} && $atom->{value} eq $lastkey) {
            $_+1 == $obj->{lastatom} and return undef;
            return $obj->{libconf}->getAtom($_+1)->{value};
        }
    }
    undef;
}

sub STORE {
    my ($obj, $key, $value) = @_;
    print "SectionWrapper - STORE - key : $key - value : $value\n";
    #ref $value eq 'HASH' or die 'trying to store anything else than hash ref';
    my $index;
    if (ref $value eq 'HASH') {
        foreach ($obj->{firstatom}+1..$obj->{lastatom}-1) {
            my $atom = $obj->{libconf}->getAtom($_);
            ($atom->{type} eq 'SECTION' && $atom->{section_name} eq $key) and $index = $_, last;
        }
        if (!defined $index) {
            $index = $obj->{libconf}->insertAtom($obj->{lastatom}, { section_name => $key,
                                                                    type => 'SECTION',
                                                                    sections => [ @{$obj->{sections}} ],
                                                                  } );
            $obj->{libconf}->insertAtom($obj->{lastatom}, { type => 'ENDSECTION',
                                                                    sections => [ @{$obj->{sections}} ],
                                                                  });
            $obj->{lastatom} += 2;
        }
        my %hash;
        tie %hash, 'Libconf::Glueconf::XF86Config::SectionWrapper', $obj->{libconf}, $index;
        %hash = %$value;
    } elsif (ref $value eq '') {
        foreach ($obj->{firstatom}+1..$obj->{lastatom}-1) {
            my $atom = $obj->{libconf}->getAtom($_);
            ($atom->{type} eq 'KEY_VALUE' && $atom->{key} eq $key) and $index = $_, last;
        }
        if (!defined $index) {
            $index = $obj->{libconf}->insertAtom($obj->{lastatom}, { key => $key,
                                                                    type => 'KEY_VALUE',
                                                                    sections => [ @{$obj->{sections}} ],
                                                                  } );
            $obj->{lastatom}++;
        }
        $obj->{libconf}->editAtom($index, { value => $value });
    } else {
        die "not implemented yet";
    }
}

sub FETCH {
    my ($obj, $key) = @_;
    print "SectionWrapper - FETCH - key : $key\n";
    my $index;
    foreach ($obj->{firstatom}+1..$obj->{lastatom}-1) { #CHANGE THIS, don't go into subsections, use better find_atom
        my $atom = $obj->{libconf}->getAtom($_);
        ($atom->{type} eq 'SECTION' && $atom->{section_name} eq $key) ||
        ($atom->{type} eq 'KEY_VALUE' && $atom->{key} eq $key) and $index = $_, last;
        ($atom->{type} eq 'VALUE' && $atom->{value} eq $key) and $index = $_, last;
    }
    defined $index or return undef;
    my $atom = $obj->{libconf}->getAtom($index);
    if ($atom->{type} eq 'SECTION') {
        my %ret;
        tie %ret, 'Libconf::Glueconf::XF86Config::SectionWrapper', $obj->{libconf}, $index;
        return \%ret;
    }
    if ($atom->{type} eq 'KEY_VALUE') {
        return $atom->{value};
    }
    if ($atom->{type} eq 'VALUE') {
        return $atom->{value};
    }
}










#            Files          File pathnames
#            ServerFlags    Server flags
#            Module         Dynamic module loading
#            InputDevice    Input device description
#            Device         Graphics device description
#            VideoAdaptor   Xv video adaptor description
#            Monitor        Monitor description
#            Modes          Video modes descriptions
#            Screen         Screen configuration
#            ServerLayout   Overall layout
#            DRI            DRI-specific configuration
#            Vendor         Vendor-specific configuration


# 1;

# my @synopsis = (
#                 {
#                  type => 'SECTION',
#                  section_name => 'Files',
#                  content => [
#                              {
#                               key => 'FontPath',
#                               type => 'KEY_VALUE',
#                              },
#                              {
#                               key => 'RGBPath',
#                               type => 'KEY_VALUE',
#                              },
#                              {
#                               key => 'ModulePath',
#                               type => 'KEY_VALUE',
#                              },
#                             ],
#                 },
#                 {
#                  type => 'SECTION',
#                  section_name => 'ServerFlags',
#                  content => [
#                              {
#                               key => 'NoTrapSignals',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'DontZap',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'DontZoom',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'DisableVidModeExtension',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'AllowNonLocalXvidtune',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'DisableModInDev',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'AllowNonLocalModInDev',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'AllowMouseOpenFail',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'VTInit',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                              },
#                              {
#                               key => 'VTSysReq',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'BlankTime',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                              },
#                              {
#                               key => 'StandbyTime',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                              },
#                              {
#                               key => 'SuspendTime',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                              },
#                              {
#                               key => 'OffTime',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                              },
#                              {
#                               key => 'Pixmap',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                              },
#                              {
#                               key => 'PC98',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'NoPM',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                              {
#                               key => 'Xinerama',
#                               type => 'KEY_VALUE',
#                               type2 => 'XF86_OPTION',
#                               value => 'BOOLEAN',
#                              },
#                             ],
#                 },
#                );

# sub new {
#     my ($pkg, $filename) = @_;
#     my $self = $pkg->Libconf::new($filename, 'XF86Config', '');
#     my $ret = Libconf::Glueconf::generateFunctions(@synopsis);
#     print " plop : $ret\n";
#     eval Libconf::Glueconf::generateFunctions(@synopsis);
#     $@ and die $@;
#     $self;
# }

1;

