package Libconf::System::DirectoryWrapper;

sub debug { Libconf::debug(@_) }

sub globize {
    my ($obj, $key) = @_;
    debug("glob : $obj->{glob}, key : $key");
    my $name = $obj->{glob};
    $name =~ s/\*/$key/g;
    debug("returning $name");
    $name;
}

sub deglobize {
    my ($obj, $filename) = @_;
    my ($glob, $directory) = ($obj->{glob}, $obj->{directory});
    debug("glob : $glob, directory : $directory, filename : $filename");
    $filename =~ s/^$directory\///;
    $glob =~ s/\*/(.*)?/g;
    $filename =~ s/$glob/$1/;
    debug("returning $filename");
    $filename;
}

sub readConf {
    my ($self) = @_;
    $self->{hash} = {};
    foreach (glob($self->{directory} . '/' . $self->{glob})) {
        my $struct = eval q(require Libconf::Glueconf::) . ucfirst($self->{template}) .
          q(; new Libconf::Glueconf::) . ucfirst($self->{template}) . "('" . $_ . "');";
        $@ and die "---- $@";
        $self->{hash}{deglobize($self, $_)} = $struct;
    }
}

sub writeConf {
    my ($self) = @_;
    $self->{hash} = {};
    foreach (glob($self->{directory} . '/' . $self->{glob})) {
        my $struct = eval q(require Libconf::Glueconf::) . ucfirst($self->{template}) .
          q(; new Libconf::Glueconf::) . ucfirst($self->{template}) . "('" . $_ . "');";
        $@ and die "---- $@";
        $self->{hash}{deglobize($self, $_)} = $struct;
    }
}

sub TIEHASH {
    my ($pkg, $directory, $glob, $template) = @_;
    debug();
    if (-e $directory) {
        -d $directory or die "$directory is not a directory";
    } else {
        mkdir $directory;
    }
    my $self = { directory => $directory,
                 glob => $glob,
                 template => $template,
               };
    readConf($self);
    bless $self, $pkg;
}

sub CLEAR {
    my ($obj) = @_;
    debug();
    unlink(glob($obj->{directory} . '/' . $obj->{glob}));
}

sub DELETE {
    my ($obj, $key) = @_;
    debug("key: $key");
    delete $obj->{hash}{$key};
#    unlink($obj->{directory} . '/' . globize($obj, $key));
}

sub FIRSTKEY {
    my ($obj) = @_;
    debug();
    my $a = keys %{$obj->{hash}};
    scalar each %{$obj->{hash}};
#    deglobize($obj, (glob($obj->{directory} . '/' . $obj->{glob}))[0]);
}

sub EXISTS {
    my ($obj, $key) = @_;
    debug("key : $key");
    exists $obj->{hash}{$key};
#    -f $obj->{directory} . "/" . globize($obj, $key);
}

sub NEXTKEY {
    my ($obj, $lastkey) = @_;
    debug("lastkey : $lastkey");
    scalar each %{$obj->{hash}};
#     my ($directory, $glob) = ($obj->{directory}, $obj->{glob});
#     my @files = glob($directory . '/' . $glob);
#     $lastkey = globize($obj, $lastkey);
#     foreach (0..@files-1) {
#         if ($files[$_] eq $lastkey) {
#             return deglobize($obj, $files[$_+1]);
#         }
#     }
#     undef;
}

sub STORE {
    my ($obj, $key, $value) = @_;
    debug("key : $key - value : $value");
    my ($directory, $glob) = ($obj->{directory}, $obj->{glob});
#    ref($value) eq 'HASH' or die "trying to store a non hash in $key";
    my $filename = $directory . '/' . globize($obj, $key);
    if (!exists $obj->{hash}{$key}) {
#        utime undef, undef, $filename; #touch replacement
        my $struct = eval q(require Libconf::Glueconf::) . ucfirst($obj->{template}) .
          q(; new Libconf::Glueconf::) . ucfirst($obj->{template}) . "('" . $filename . "');";
        $@ and die $@;
        $obj->{hash}{$key} = $struct;
    }
    $obj->{hash}{$key} = $value;
#     %$struct = %$value;
#     $struct->{libconf}->writeConf($filename); #FIXME, when should we write to the file?
}

sub FETCH {
    my ($obj, $key) = @_;
    debug("key : $key");
    $obj->{hash}{$key};
#    my ($directory, $glob) = ($obj->{directory}, $obj->{glob});
#    my $filename = $directory . '/' . globize($obj, $key);
#    new Libconf::Glueconf::Shell($filename);
}

1;
